@@ -1,5 +1,5 @@
 """Init file """
-__version__= '0.0.1'
+__version__= '0.0.13'
 
 import sys, os
 
@@ -11,4 +11,7 @@ import dreem.vector.main as vectoring
 import dreem.cluster.main as clustering
 import dreem.aggregate.main as aggregation
 import dreem.draw.main as drawer
+from dreem.draw.study import Study
 
+import warnings
+warnings.simplefilter(action='ignore', category=FutureWarning)
