@@ -1,33 +1,29 @@
-import click
 from pipeline import run
-from click_option_group import optgroup
 from dreem.util.cli import *
 
+
 @click.command()
-
 @optgroup.group('I/O')
-@opto_out_dir
-@fasta
-@fastq1
-@fastq2
-@input_dir
-@samples
-@sample
-@clustering_file
-@opti_library    
-
+@opt_top_dir
+@arg_fasta
+@opt_fastqs
+@opt_fastqi
+@opt_fastq1
+@opt_fastq2
+@opt_library
+@opt_phred_enc
+@opt_min_phred
+@opt_rerun
 @optgroup.group('Selection')
-@coords
-@primers
-@fill
-@parallel
-
+@opt_coords
+@opt_primers
+@opt_fill
+@opt_parallel
 @optgroup.group('Demultiplexing')
 @demultiplexing
 @barcode_start
 @barcode_length
 @max_barcode_mismatches
-
 @optgroup.group('Clustering')
 @clustering
 @max_clusters
@@ -39,8 +35,7 @@ from dreem.util.cli import *
 @min_reads
 @convergence_cutoff
 @num_runs
-@n_cpus
-
+@opt_cpus
 @optgroup.group('Aggregation')
 @rnastructure_path
 @rnastructure_temperature
@@ -50,14 +45,13 @@ from dreem.util.cli import *
 @rnastructure_dms_max_paired_value
 @rnastructure_partition
 @rnastructure_probability
-@poisson
+
 
 @optgroup.group('Misc')
 @verbose
+def cli(*args, **kwargs):
+    run(*args, **kwargs)
 
-def cli(**args):
-    run(**args)
 
 if __name__ == '__main__':
-    run()
-
+    cli()
