@@ -6,10 +6,10 @@ from dreem.util.cli import *
 @click.command()
 
 @optgroup.group('I/O')
-@opti_library
+@opt_library
 @fastq1
 @fastq2
-@opto_out_dir
+@opt_top_dir
 
 @optgroup.group('Demultiplexing')
 @barcode_start
@@ -29,4 +29,4 @@ def cli(**args):
     run(**args)
 
 if __name__ == '__main__':
-    cli()
\ No newline at end of file
+    cli()
