@@ -16,82 +16,67 @@ class Study(object):
 
     attr_list = ['name','samples']
 
-    def __init__(self, data=None, samples=None, min_cov_bases=0, filter_by='sample') -> None:
+    def __init__(self, data=None, min_cov_bases=0, filter_by='sample') -> None:
         """Creates a Study object.
 
         Args:
-            data (dict or list[dict], optional): A dictionary or list of dictionaries containing the data to be loaded. One dictionary corresponds to a fastq file. Defaults to None.
-            samples (List[str], optional): List of samples to load. Defaults to None.
+            data (dict or list[dict] or pandas.DataFrame, optional): Data to use. Can be a dictionary or list of dictionaries containing DREEM-output jsons, or directly a pandas dataframe. Defaults to None.
             min_cov_bases (int, optional): Minimum number of base coverage for a row to be filtered-in. Defaults to 0.
-            filter_by (str, optional): Filter rows by sample or study. When filtered by study, if a row passes the filter, rows with the same 'construct', 'section' and 'cluster' fields for all other samples have a sufficient base coverage. Defaults to 'sample'.            
+            filter_by (str, optional): Filter rows by sample or study. When filtered by study, if a row passes the filter, rows with the same 'reference', 'section' and 'cluster' fields for all other samples have a sufficient base coverage. Defaults to 'sample'.            
 
         Example:
-            >>> study = Study(data = {'sample':'mysample',{'construct1': {'section1': {'cluster1': {'mut_bases': [100], 'cov_bases': [1000]}}}}},
-                              samples=['A1', 'B2', 'B3'], 
+            >>> study = Study(data = {'sample':'mysample',{'reference1': {'section1': {'cluster1': {'mut_bases': [100], 'cov_bases': [1000]}}}}},
                               min_cov_bases=1000, 
-                              filter_by='study')
+                              filter_by='sample')
         """
-        self.samples = samples
         if data is not None:
-            self.df = pd.DataFrame()
-            if type(data) is not list:
-                data = [data]
-            for sample in data:
-                self.df = pd.concat([self.df, pd.DataFrame(flatten_json(sort_dict(sample)))], axis=0)
-            self.set_df(self.df, min_cov_bases=min_cov_bases, filter_by=filter_by, samples=samples)
+            
+            df = pd.DataFrame()              
+            
+            # If data is a list of json, concatenate them into a single dataframe
+            if type(data) is not pd.DataFrame:
+                print('Turning data into a dataframe...')
+                
+                if type(data) is not list:
+                    data = [data]
+                    
+                for sample in data:
+                    print(sample['sample'], end='... ')
+                    df = pd.concat([df, pd.DataFrame(flatten_json(sort_dict(sample)))], axis=0)
+                
+                print('Done.')
+            
+            # Use the dataframe (loaded or created from json)
+            print('Setting dataframe...')
+            self.set_df(df, min_cov_bases=min_cov_bases, filter_by=filter_by)
+            print('Done.')
+            
         else:
             self.df = None
 
-    @classmethod
-    def from_dict(cls, di:dict):
-        """Set attributes of this Study object from a dictionary.
-
-        Args:
-            di (dict): a dictionary containing keys such as ['name','description','samples','label','conditions'].
-
-        Returns:
-            Study: a study object.
-
-        Example:
-        >>> di = {'name':'temperature','samples':['A1','B2','B3']}
-        >>> study = Study().from_dict(di)
-        >>> print(study.name, study.samples)
-        temperature ['A1', 'B2', 'B3']
-        """
-        for attr in cls.attr_list:
-            try: 
-                di[attr]
-            except: 
-                di[attr]=None
-        return cls(di['name'], di['samples'])
-
-    def set_df(self, df, min_cov_bases=0, filter_by='sample', samples=None):
-        df.reset_index(inplace=True, drop=True)
-        self.df = df
-        for col in [ 'mut_bases', 'info_bases','del_bases','ins_bases','cov_bases','mut_rates'] + \
-            [c for c in self.df.columns.tolist() if (c.startswith('mod_bases') or c.startswith('poisson'))]:
-            if type(self.df[col].iloc[0]) == str:
-                self.df[col] = self.df[col].apply(lambda x: np.array([float(b) for b in x[1:-1].replace('\n',' ').replace(',',' ').split(' ') if b != '']))
-
-        if not 'worst_cov_bases' in self.df.columns:
-            self.df['worst_cov_bases'] = self.df['cov_bases'].apply(lambda x: min(x))
-
-        self.df = manipulator.get_df(df=self.df, sample=samples, min_cov_bases=min_cov_bases, selected_cols=False)
-        if filter_by == 'study':
-            self.filter_by_study(inplace=True)
+    
+    def set_df(self, df, min_cov_bases=0, filter_by='sample'):
         
-        for attr in ['sample','construct']:
+        self.df = df.reset_index(drop=True)
+                
+        self.df = self.df[self.df['worst_cov_bases'] >= min_cov_bases]
+        
+        if filter_by == 'study':
+            self.filter_by_study()
+        
+        for attr in ['sample','reference']:
             self.df[attr] = self.df[attr].astype(str)
         
         for attr in ['section','cluster']:
             if attr not in self.df.columns:
                 self.df[attr] = 0
+        
+        self.df['deltaG'] = self.df['deltaG'].apply(lambda x: 0.0 if x == 'void' else float(x))
     
-    def filter_by_study(self, inplace=False):
-        df = self.df.groupby(['construct', 'section', 'cluster']).filter(lambda x: len(self.df['sample'].unique()) == len(x['sample'].unique()))
-        if inplace:
-            self.df = df
-        return df.copy()
+    
+    def filter_by_study(self):
+        df = self.df.groupby(['reference', 'section', 'cluster']).filter(lambda x: len(self.df['sample'].unique()) == len(x['sample'].unique()))
+        self.df = df
 
     def get_df(self, **kwargs):
         return manipulator.get_df(self.df, **kwargs)
@@ -99,25 +84,22 @@ class Study(object):
     def get_samples(self):
         return self.df['sample'].unique()
 
-    def get_constructs(self, sample:str):
-        return self.df[self.df['sample'] == sample]['construct'].unique()
+    def get_references(self, sample:str):
+        return self.df[self.df['sample'] == sample]['reference'].unique()
 
-    def get_sections(self, sample:str, construct:str):
-        return self.df[(self.df['sample'] == sample) & (self.df['construct'] == construct)]['section'].unique()
+    def get_sections(self, sample:str, reference:str):
+        return self.df[(self.df['sample'] == sample) & (self.df['reference'] == reference)]['section'].unique()
 
-    def get_clusters(self, sample:str, construct:str, section:str):
-        return self.df[(self.df['sample'] == sample) & (self.df['construct'] == construct)& (self.df['section'] == section)]['cluster'].unique()
+    def get_clusters(self, sample:str, reference:str, section:str):
+        return self.df[(self.df['sample'] == sample) & (self.df['reference'] == reference)& (self.df['section'] == section)]['cluster'].unique()
        
-    def load_studies(studies_file_path:str):
-        return load_studies(studies_file_path)
-
 
     def mutation_fraction(self, **kwargs)->dict:
         """Plot the mutation rates as histograms.
 
         Args:
             sample (list, int, str, optional): Filter rows by sample (list of samples or just a sample). Defaults to None.
-            construct (list, int, str, optional): Filter rows by construct (list of constructs or just a construct). Defaults to None.
+            reference (list, int, str, optional): Filter rows by reference (list of references or just a reference). Defaults to None.
             section (list, int, str, optional): Filter rows by section (list of sections or just a section). Defaults to None.
             cluster (list, int, str, optional): Filter rows by cluster (list of clusters or just a cluster). Defaults to None.
             base_index (list, int, str, optional): Filter per-base attributes (mut_rates, sequence, etc) by base index. Can be a unique sequence in the row's sequence, a list of indexes or a single index. Defaults to None.
@@ -137,11 +119,11 @@ class Study(object):
         return plotter.mutation_fraction(manipulator.get_df(self.df, **{k:v for k,v in kwargs.items() if k in list(self.df.columns)+ list(manipulator.get_df.__code__.co_varnames)}), **{k:v for k,v in kwargs.items() if k in plotter.mutation_fraction.__code__.co_varnames})
 
     def deltaG_vs_mut_rates(self, **kwargs)->dict:
-        """Plot the mutation rate of each paired-expected base of the ROI for each construct of a sample, w.r.t the deltaG estimation.
+        """Plot the mutation rate of each paired-expected base of the ROI for each reference of a sample, w.r.t the deltaG estimation.
 
         Args:
             sample (list, int, str, optional): Filter rows by sample (list of samples or just a sample). Defaults to None.
-            construct (list, int, str, optional): Filter rows by construct (list of constructs or just a construct). Defaults to None.
+            reference (list, int, str, optional): Filter rows by reference (list of references or just a reference). Defaults to None.
             section (list, int, str, optional): Filter rows by section (list of sections or just a section). Defaults to None.
             cluster (list, int, str, optional): Filter rows by cluster (list of clusters or just a cluster). Defaults to None.
             min_cov_bases (int, optional): Filter rows by a minimum threshold for base coverage. Defaults to 0.
@@ -163,10 +145,10 @@ class Study(object):
 
     
     def exp_variable_across_samples(self, **kwargs)->dict:
-        """Plot the mutation rate of each paired-expected base of the ROI for each construct of a sample, w.r.t the deltaG estimation.
+        """Plot the mutation rate of each paired-expected base of the ROI for each reference of a sample, w.r.t the deltaG estimation.
 
         Args:
-            construct (str): Construct of your row.
+            reference (str): reference of your row.
             experimental_variable (str): x axis column value, must be a per-sample attribute.
             section (list, int, str, optional): Filter rows by section (list of sections or just a section). Defaults to None.
             cluster (str): Cluster of your row.
@@ -191,7 +173,7 @@ class Study(object):
 
         Args:
             sample (list, int, str, optional): Filter rows by sample (list of samples or just a sample). Defaults to None.
-            construct (list, int, str, optional): Filter rows by construct (list of constructs or just a construct). Defaults to None.
+            reference (list, int, str, optional): Filter rows by reference (list of references or just a reference). Defaults to None.
             section (list, int, str, optional): Filter rows by section (list of sections or just a section). Defaults to None.
             cluster (list, int, str, optional): Filter rows by cluster (list of clusters or just a cluster). Defaults to None.
             min_cov_bases (int, optional): Filter rows by a minimum threshold for base coverage. Defaults to 0.
@@ -215,8 +197,8 @@ class Study(object):
         """Plot the mutation rate difference between two mutation profiles.
         sample1: sample of the first mutation profile.
         sample2: sample of the second mutation profile.
-        construct1: construct of the first mutation profile.
-        construct2: construct of the second mutation profile.
+        reference1: reference of the first mutation profile.
+        reference2: reference of the second mutation profile.
         section1: section of the first mutation profile.
         section2: section of the second mutation profile.
         cluster1: cluster of the first mutation profile.
@@ -241,11 +223,11 @@ class Study(object):
         return plotter.mutation_fraction_delta(pd.concat([df1, df2]).reset_index(drop=True), **{k:v for k,v in kwargs.items() if k in plotter.mutation_fraction_delta.__code__.co_varnames})
 
     def base_coverage(self, **kwargs):
-        """Plot the base coverage of several constructs in a sample.
+        """Plot the base coverage of several references in a sample.
 
         Args:
-            samp (str): Sample of your rows.
-            constructs (List[str]): Constructs of your rows.
+            sample (str): Sample of your rows.
+            reference (List[str]): references of your rows.
             section (str): Region of your row.
             cluster (int, optional): Cluster of your row. Defaults to 0. 
             index (_type_, optional): Indexes to plot. Defaults to ``'all'``. Can be a series of 0-indexes (ex: [43,44,45,48]), 'roi', 'all', or a unique sequence (ex: 'ATTAC')
@@ -263,22 +245,3 @@ class Study(object):
         """
         return 0# plotter.base_coverage(self._df, **kwargs)
 
-
-
-def load_studies(studies_file_path:str)->dict[str:Study]:
-    """Read formatted file with samples, and turn it into a dataframe containing studies.
-
-    Args:
-        studies_file_path (str): path+title of the csv file containing the samples.
-
-    Returns:
-        (pd.DataFrame): studies of the csv file, indexed by study.
-    """
-
-    studies_dict, studies_data = {}, pd.read_csv(studies_file_path)
-
-    for col in studies_data.groupby('name')[Study.attr_list]:
-        solo_item = lambda x: x[0] if len(set(x)) == 1 else x  
-        studies_dict[col[0]] = {attr: solo_item(list(col[1][attr])) for attr in (Study.attr_list)} 
-
-    return {k:Study.from_dict(v) for k,v in studies_dict.items()}
